@@ -38,11 +38,6 @@
 /* Some various global definitions.  */
 
 /* Name of file to use for interacting with user.  */
-#if MSDOS
-# define TTY_NAME "con"
-#else
-# define TTY_NAME "/dev/tty"
-#endif
 
 /* GLOBAL is defined to empty in tar.c only, and left alone in other *.c
    modules.  Here, we merely set it to "extern" if it is not already set.
@@ -160,11 +155,12 @@ GLOBAL int check_links_option;
 /* Patterns that match file names to be excluded.  */
 GLOBAL struct exclude *excluded;
 
+/* Exclude directories containing a cache directory tag. */
+GLOBAL bool exclude_caches_option;
+
 /* Specified file containing names to work on.  */
 GLOBAL const char *files_from_option;
 
-GLOBAL bool force_local_option;
-
 /* Specified value to be put into tar file in place of stat () results, or
    just -1 if such an override should not take place.  */
 GLOBAL gid_t group_option;
@@ -505,6 +501,7 @@ void print_for_mkdir (char *, int, mode_t);
 void print_header (struct tar_stat_info *, off_t);
 void read_and (void (*) (void));
 enum read_header read_header (bool);
+enum read_header tar_checksum (union block *header);
 void skip_file (off_t);
 void skip_member (void);
 
@@ -524,7 +521,15 @@ enum remove_option
 {
   ORDINARY_REMOVE_OPTION,
   RECURSIVE_REMOVE_OPTION,
-  WANT_DIRECTORY_REMOVE_OPTION
+
+  /* FIXME: The following value is never used. It seems to be intended
+     as a placeholder for a hypothetical option that should instruct tar
+     to recursively remove subdirectories in purge_directory(),
+     as opposed to the functionality of --recursive-unlink
+     (RECURSIVE_REMOVE_OPTION value), which removes them in
+     prepare_to_extract() phase. However, with the addition of more
+     meta-info to the incremental dumps, this should become unnecessary */
+  WANT_DIRECTORY_REMOVE_OPTION  
 };
 int remove_any_file (const char *, enum remove_option);
 bool maybe_backup_file (const char *, int);
@@ -678,13 +683,10 @@ bool sys_compare_gid (struct stat *a, struct stat *b);
 bool sys_file_is_archive (struct tar_stat_info *p);
 bool sys_compare_links (struct stat *link_data, struct stat *stat_data);
 int sys_truncate (int fd);
-void sys_reset_uid_gid (void);
 pid_t sys_child_open_for_compress (void);
 pid_t sys_child_open_for_uncompress (void);
-void sys_reset_uid_gid (void);
 size_t sys_write_archive_buffer (void);
 bool sys_get_archive_stat (void);
-void sys_reset_uid_gid (void);
 
 /* Module compare.c */
 void report_difference (struct tar_stat_info *st, const char *message, ...);
